# -*- coding: UTF-8 -*-
#/**
# * Software Name : pycrate
# * Version : 0.4
# *
# * Copyright 2018. Benoit Michau. ANSSI. P1sec.
# *
# * This library is free software; you can redistribute it and/or
# * modify it under the terms of the GNU Lesser General Public
# * License as published by the Free Software Foundation; either
# * version 2.1 of the License, or (at your option) any later version.
# *
# * This library is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# * Lesser General Public License for more details.
# *
# * You should have received a copy of the GNU Lesser General Public
# * License along with this library; if not, write to the Free Software
# * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
# * MA 02110-1301  USA
# *
# *--------------------------------------------------------
# * File Name : pycrate_csn1dir/packet_uplink_assignment_message_content.py
# * Created : 2018-11-21
# * Authors : Benoit Michau
# *--------------------------------------------------------
#*/
# specification: TS 44.060 - d60
# section: 11.2.29 Packet Uplink Assignment
# top-level object: Packet Uplink Assignment message content

# external references
from pycrate_csn1dir.starting_frame_number_description_ie import starting_frame_number_description_ie
from pycrate_csn1dir.padding_bits import padding_bits
from pycrate_csn1dir.frequency_parameters_ie import frequency_parameters_ie
from pycrate_csn1dir.dynamic_allocation_3_ie import dynamic_allocation_3_ie
from pycrate_csn1dir.pulse_format_ie import pulse_format_ie
from pycrate_csn1dir.packet_request_reference_ie import packet_request_reference_ie
from pycrate_csn1dir.egprs_level_ie import egprs_level_ie
from pycrate_csn1dir.packet_timing_advance_ie import packet_timing_advance_ie
from pycrate_csn1dir.egprs_modulation_and_coding_scheme_ie import egprs_modulation_and_coding_scheme_ie
from pycrate_csn1dir.egprs_window_size_ie import egprs_window_size_ie
from pycrate_csn1dir.global_tfi_ie import global_tfi_ie
from pycrate_csn1dir.dual_carrier_frequency_parameters_ie import dual_carrier_frequency_parameters_ie
from pycrate_csn1dir.pdch_pairs_description_ie import pdch_pairs_description_ie
from pycrate_csn1dir.compact_reduced_ma_ie import compact_reduced_ma_ie

# code automatically generated by pycrate_csn1
# change object type with type=CSN1T_BSTR (default type is CSN1T_UINT) in init
# add dict for value interpretation with dic={...} in CSN1Bit init
# add dict for key interpretation with kdic={...} in CSN1Alt init

from pycrate_csn1.csnobj import *

access_technologies_request_struct = CSN1List(name='access_technologies_request_struct', list=[
  CSN1Bit(name='access_technology_type', bit=4),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1SelfRef()])})])

dynamic_allocation_struct = CSN1List(name='dynamic_allocation_struct', list=[
  CSN1Bit(name='extended_dynamic_allocation'),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='p0', bit=4),
    CSN1Bit(name='pr_mode')])}),
  CSN1Bit(name='usf_granularity'),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='uplink_tfi_assignment', bit=5)])}),
  CSN1Val(name='', val='0'),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Ref(name='tbf_starting_time', obj=starting_frame_number_description_ie)])}),
  CSN1Alt(alt={
    '0': ('', [
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn0', bit=3)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn1', bit=3)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn2', bit=3)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn3', bit=3)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn4', bit=3)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn5', bit=3)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn6', bit=3)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn7', bit=3)])})]),
    '1': ('', [
    CSN1Bit(name='alpha', bit=4),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn0', bit=3),
      CSN1Bit(name='gamma_tn0', bit=5)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn1', bit=3),
      CSN1Bit(name='gamma_tn1', bit=5)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn2', bit=3),
      CSN1Bit(name='gamma_tn2', bit=5)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn3', bit=3),
      CSN1Bit(name='gamma_tn3', bit=5)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn4', bit=3),
      CSN1Bit(name='gamma_tn4', bit=5)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn5', bit=3),
      CSN1Bit(name='gamma_tn5', bit=5)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn6', bit=3),
      CSN1Bit(name='gamma_tn6', bit=5)])}),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf_tn7', bit=3),
      CSN1Bit(name='gamma_tn7', bit=5)])})])})])

additional_pfcs_struct = CSN1List(name='additional_pfcs_struct', list=[
  CSN1Bit(name='uplink_tfi_assignment', bit=5),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='npm_transfer_time', bit=5)])}),
  CSN1Bit(name='pfi', bit=7)])

rlc_entity_struct = CSN1List(name='rlc_entity_struct', list=[
  CSN1Bit(name='uplink_tfi_assignment', bit=5),
  CSN1Bit(name='rlc_mode'),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Ref(name='egprs_window_size', obj=egprs_window_size_ie)])}),
  CSN1Bit(name='pfi', bit=7)])

assignment_info_struct = CSN1List(name='assignment_info_struct', list=[
  CSN1Bit(name='assignment_type', bit=2),
  CSN1Bit(name='carrier_id')])

extension = CSN1Val(name='extension', val='null')

single_block_allocation_struct = CSN1List(name='single_block_allocation_struct', list=[
  CSN1Bit(name='timeslot_number', bit=3),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='alpha', bit=4),
    CSN1Bit(name='gamma_tn', bit=5)])}),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='p0', bit=4),
    CSN1Val(name='', val='0'),
    CSN1Bit(name='pr_mode')])}),
  CSN1Ref(name='tbf_starting_time', obj=starting_frame_number_description_ie)])

dynamic_allocation_2_struct = CSN1List(name='dynamic_allocation_2_struct', list=[
  CSN1Bit(name='extended_dynamic_allocation'),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='p0_c1', bit=4),
    CSN1Bit(name='pr_mode_c1'),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='p0_c2', bit=4),
      CSN1Bit(name='pr_mode_c2')])})])}),
  CSN1Bit(name='usf_granularity'),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='uplink_tfi_assignment', bit=5)])}),
  CSN1Alt(alt={
    '0': ('', [
    CSN1Bit(name='n_usf', bit=4),
    CSN1Alt(num=([1], lambda x: x + 1), alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='usf', bit=3)])})]),
    '1': ('', [
    CSN1Bit(name='alpha_c1', bit=4),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='alpha_c2', bit=4)])}),
    CSN1Alt(alt={
      '0': ('', [
      CSN1Bit(name='n_ts', bit=4),
      CSN1Alt(num=([1], lambda x: x + 1), alt={
        '0': ('', []),
        '1': ('', [
        CSN1Bit(name='usf', bit=3),
        CSN1Bit(name='gamma', bit=5)])})]),
      '1': ('', [
      CSN1Bit(name='n_pairs', bit=3),
      CSN1Alt(num=([1], lambda x: x + 1), alt={
        '0': ('', []),
        '1': ('', [
        CSN1Bit(name='usf', bit=3),
        CSN1Bit(name='gamma', bit=5)])}),
      CSN1Alt(alt={
        '0': ('', []),
        '1': ('', [
        CSN1Alt(num=([-1, 1], lambda x: x + 1), alt={
          '0': ('', []),
          '1': ('', [
          CSN1Bit(name='usf_2', bit=3),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='gamma', bit=5)])})])})])})])})])})])

dlmc_ul_carrier_info_struct = CSN1Alt(name='dlmc_ul_carrier_info_struct', alt={
  '00': ('', []),
  '01': ('', []),
  '10': ('', [
  CSN1Alt(alt={
    '0': ('', [
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='uplink_timeslot_allocation', bit=8)])})]),
    '1': ('', [
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Bit(name='uplink_pdch_pairs_dlmc', bit=8)])})])}),
  CSN1Ref(name='dynamic_allocation_3', obj=dynamic_allocation_3_ie),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Ref(name='uplink_rlc_entity_2', obj=rlc_entity_struct),
    CSN1Alt(alt={
      '0': ('', []),
      '1': ('', [
      CSN1Ref(name='uplink_rlc_entity_3', obj=rlc_entity_struct)])})])}),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1List(num=-1, list=[
      CSN1Val(name='', val='1'),
      CSN1Ref(name='emsr_additional_pfcs_1', obj=additional_pfcs_struct)]),
    CSN1Val(name='', val='0'),
    CSN1List(num=-1, list=[
      CSN1Val(name='', val='1'),
      CSN1Ref(name='emsr_additional_pfcs_2', obj=additional_pfcs_struct)]),
    CSN1Val(name='', val='0'),
    CSN1List(num=-1, list=[
      CSN1Val(name='', val='1'),
      CSN1Ref(name='emsr_additional_pfcs_3', obj=additional_pfcs_struct)]),
    CSN1Val(name='', val='0')])}),
  CSN1Bit(name='primary_tsc_set')]),
  '11': ('', [
  CSN1Bit(bit=-1)])})

multi_block_allocation_struct = CSN1List(name='multi_block_allocation_struct', list=[
  CSN1Bit(name='timeslot_number', bit=3),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='alpha', bit=4),
    CSN1Bit(name='gamma_tn', bit=5)])}),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='p0', bit=4),
    CSN1Val(name='', val='0'),
    CSN1Bit(name='pr_mode')])}),
  CSN1Ref(name='tbf_starting_time', obj=starting_frame_number_description_ie),
  CSN1Bit(name='number_of_radio_blocks_allocated', bit=2)])

packet_uplink_assignment_message_content = CSN1List(name='packet_uplink_assignment_message_content', list=[
  CSN1Bit(name='page_mode', bit=2),
  CSN1Alt(alt={
    '0': ('', []),
    '1': ('', [
    CSN1Bit(name='persistence_level', bit=16)])}),
  CSN1List(list=[
    CSN1Alt(alt={
      '0': ('', [
      CSN1Ref(name='global_tfi', obj=global_tfi_ie)]),
      '10': ('', [
      CSN1Bit(name='tlli_g_rnti', bit=32)]),
      '110': ('', [
      CSN1Bit(name='tqi', bit=16)]),
      '111': ('', [
      CSN1Ref(name='packet_request_reference', obj=packet_request_reference_ie)])}),
    CSN1Alt(alt={
      '0': ('', [
      CSN1List(list=[
        CSN1Bit(name='channel_coding_command', bit=2),
        CSN1Bit(name='tlli_block_channel_coding'),
        CSN1Ref(name='packet_timing_advance', obj=packet_timing_advance_ie),
        CSN1Alt(alt={
          '0': ('', []),
          '1': ('', [
          CSN1Ref(name='frequency_parameters', obj=frequency_parameters_ie)])}),
        CSN1Alt(alt={
          '00': ('', [
          CSN1Ref(obj=extension)]),
          '01': ('', [
          CSN1Ref(name='dynamic_allocation', obj=dynamic_allocation_struct)]),
          '10': ('', [
          CSN1Ref(name='single_block_allocation', obj=single_block_allocation_struct)])}),
        CSN1Alt(alt={
          '0': ('', [
          CSN1Bit(bit=-1)]),
          '1': ('', [
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='packet_extended_timing_advance', bit=2)])}),
          CSN1Alt(alt={
            '0': ('', [
            CSN1Bit(bit=-1)]),
            '1': ('', [
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Bit(name='g_rnti_extension', bit=4)])}),
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Bit(name='rb_id', bit=5)])}),
            CSN1Alt(alt={
              '0': ('', [
              CSN1Bit(bit=-1)]),
              '1': ('', [
              CSN1Alt(alt={
                '0': ('', []),
                '1': ('', [
                CSN1Bit(name='pfi', bit=7)])}),
              CSN1Alt(alt={
                '0': ('', []),
                '1': ('', [
                CSN1Bit(name='rlc_mode')])}),
              CSN1Alt(alt={
                '0': ('', [
                CSN1Bit(bit=-1)]),
                '1': ('', [
                CSN1Bit(name='primary_tsc_set'),
                CSN1Ref(obj=padding_bits)]),
                None: ('', [])})]),
              None: ('', [])})]),
            None: ('', [])})]),
          None: ('', [])})])]),
      '1': ('', [
      CSN1Alt(alt={
        '00': ('', [
        CSN1List(list=[
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='contention_resolution_tlli', bit=32)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Ref(name='compact_reduced_ma', obj=compact_reduced_ma_ie)])}),
          CSN1Ref(name='egprs_channel_coding_command', obj=egprs_modulation_and_coding_scheme_ie),
          CSN1Bit(name='resegment'),
          CSN1Ref(name='egprs_window_size', obj=egprs_window_size_ie),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Ref(name='access_technologies_request', obj=access_technologies_request_struct)])}),
          CSN1Bit(name='arac_retransmission_request'),
          CSN1Bit(name='tlli_block_channel_coding'),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='bep_period2', bit=4)])}),
          CSN1Ref(name='packet_timing_advance', obj=packet_timing_advance_ie),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='packet_extended_timing_advance', bit=2)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Ref(name='frequency_parameters', obj=frequency_parameters_ie)])}),
          CSN1Alt(alt={
            '00': ('', [
            CSN1Ref(obj=extension)]),
            '01': ('', [
            CSN1Ref(name='dynamic_allocation', obj=dynamic_allocation_struct)]),
            '10': ('', [
            CSN1Ref(name='multi_block_allocation', obj=multi_block_allocation_struct)])}),
          CSN1Alt(alt={
            '0': ('', [
            CSN1Bit(bit=-1)]),
            '1': ('', [
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Bit(name='g_rnti_extension', bit=4)])}),
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Bit(name='rb_id', bit=5)])}),
            CSN1Alt(alt={
              '0': ('', [
              CSN1Bit(bit=-1)]),
              '1': ('', [
              CSN1Alt(alt={
                '0': ('', []),
                '1': ('', [
                CSN1Bit(name='pfi', bit=7)])}),
              CSN1Alt(alt={
                '0': ('', []),
                '1': ('', [
                CSN1Bit(name='rlc_mode')])}),
              CSN1Alt(alt={
                '0': ('', [
                CSN1Bit(bit=-1)]),
                '1': ('', [
                CSN1Alt(alt={
                  '0': ('', []),
                  '1': ('', [
                  CSN1Bit(name='npm_transfer_time', bit=5)])}),
                CSN1Alt(alt={
                  '0': ('', [
                  CSN1Bit(bit=-1)]),
                  '1': ('', [
                  CSN1Bit(name='indication_of_upper_layer_pdu_start_for_rlc_um'),
                  CSN1Bit(name='enhanced_flexible_timeslot_assignment'),
                  CSN1Alt(alt={
                    '0': ('', [
                    CSN1Bit(name='emst_nw_capability')]),
                    '1': ('', [
                    CSN1Ref(name='rlc_entity_2', obj=rlc_entity_struct),
                    CSN1Alt(alt={
                      '0': ('', []),
                      '1': ('', [
                      CSN1Ref(name='rlc_entity_3', obj=rlc_entity_struct)])})])}),
                  CSN1Alt(alt={
                    '0': ('', [
                    CSN1Bit(bit=-1)]),
                    '1': ('', [
                    CSN1List(num=-1, list=[
                      CSN1Val(name='', val='1'),
                      CSN1Ref(name='emsr_additional_pfcs_1', obj=additional_pfcs_struct)]),
                    CSN1Val(name='', val='0'),
                    CSN1List(num=-1, list=[
                      CSN1Val(name='', val='1'),
                      CSN1Ref(name='emsr_additional_pfcs_2', obj=additional_pfcs_struct)]),
                    CSN1Val(name='', val='0'),
                    CSN1List(num=-1, list=[
                      CSN1Val(name='', val='1'),
                      CSN1Ref(name='emsr_additional_pfcs_3', obj=additional_pfcs_struct)]),
                    CSN1Val(name='', val='0'),
                    CSN1Alt(alt={
                      '0': ('', [
                      CSN1Bit(bit=-1)]),
                      '1': ('', [
                      CSN1Bit(name='primary_tsc_set'),
                      CSN1Ref(obj=padding_bits)]),
                      None: ('', [])})]),
                    None: ('', [])})]),
                  None: ('', [])})]),
                None: ('', [])})]),
              None: ('', [])})]),
            None: ('', [])})])]),
        '01': ('', [
        CSN1List(list=[
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='contention_resolution_tlli', bit=32)])}),
          CSN1Bit(name='resegment'),
          CSN1Ref(name='assignment_info', obj=assignment_info_struct),
          CSN1Ref(name='egprs_window_size', obj=egprs_window_size_ie),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Ref(name='access_technologies_request', obj=access_technologies_request_struct)])}),
          CSN1Bit(name='arac_retransmission_request'),
          CSN1Bit(name='tlli_block_channel_coding'),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='bep_period2', bit=4)])}),
          CSN1Ref(name='packet_timing_advance', obj=packet_timing_advance_ie),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='packet_extended_timing_advance', bit=2)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='rtti_usf_mode'),
            CSN1Ref(name='pdch_pairs_description', obj=pdch_pairs_description_ie)])}),
          CSN1Ref(name='dynamic_allocation_2', obj=dynamic_allocation_2_struct),
          CSN1Ref(name='egprs_channel_coding_command', obj=egprs_modulation_and_coding_scheme_ie),
          CSN1Alt(alt={
            '00': ('', []),
            '01': ('', [
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Ref(name='frequency_parameters_c1', obj=frequency_parameters_ie)])}),
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Ref(name='frequency_parameters_c2', obj=frequency_parameters_ie)])})]),
            '10': ('', [
            CSN1Ref(name='dual_carrier_frequency_parameters', obj=dual_carrier_frequency_parameters_ie)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='pfi', bit=7)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='rlc_mode')])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='npm_transfer_time', bit=5)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Bit(name='reported_timeslots_c1', bit=8),
              CSN1Alt(alt={
                '0': ('', []),
                '1': ('', [
                CSN1Bit(name='reported_timeslots_c2', bit=8)])}),
              CSN1Bit(name='tsh', bit=2)])})])}),
          CSN1Ref(name='uplink_egprs_level', obj=egprs_level_ie),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Ref(name='pulse_format', obj=pulse_format_ie)])}),
          CSN1Alt(alt={
            '0': ('', [
            CSN1Bit(bit=-1)]),
            '1': ('', [
            CSN1Bit(name='indication_of_upper_layer_pdu_start_for_rlc_um'),
            CSN1Bit(name='enhanced_flexible_timeslot_assignment'),
            CSN1Alt(alt={
              '0': ('', [
              CSN1Bit(name='emst_nw_capability')]),
              '1': ('', [
              CSN1Ref(name='rlc_entity_2', obj=rlc_entity_struct),
              CSN1Alt(alt={
                '0': ('', []),
                '1': ('', [
                CSN1Ref(name='rlc_entity_3', obj=rlc_entity_struct)])})])}),
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Bit(name='mtti_uplink_assignment_c1')])}),
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Bit(name='mtti_uplink_assignment_c2')])}),
            CSN1Alt(alt={
              '0': ('', [
              CSN1Bit(bit=-1)]),
              '1': ('', [
              CSN1List(num=-1, list=[
                CSN1Val(name='', val='1'),
                CSN1Ref(name='emsr_additional_pfcs_1', obj=additional_pfcs_struct)]),
              CSN1Val(name='', val='0'),
              CSN1List(num=-1, list=[
                CSN1Val(name='', val='1'),
                CSN1Ref(name='emsr_additional_pfcs_2', obj=additional_pfcs_struct)]),
              CSN1Val(name='', val='0'),
              CSN1List(num=-1, list=[
                CSN1Val(name='', val='1'),
                CSN1Ref(name='emsr_additional_pfcs_3', obj=additional_pfcs_struct)]),
              CSN1Val(name='', val='0'),
              CSN1Alt(alt={
                '0': ('', [
                CSN1Bit(bit=-1)]),
                '1': ('', [
                CSN1Bit(name='primary_tsc_set'),
                CSN1Ref(obj=padding_bits)]),
                None: ('', [])})]),
              None: ('', [])})]),
            None: ('', [])})])]),
        '10': ('', [
        CSN1List(trunc=True, list=[
          CSN1Bit(name='resegment'),
          CSN1List(num=-1, list=[
            CSN1Val(name='', val='1'),
            CSN1Ref(name='dlmc_ul_carrier_info', obj=dlmc_ul_carrier_info_struct)]),
          CSN1Val(name='', val='0'),
          CSN1Ref(name='egprs_window_size', obj=egprs_window_size_ie),
          CSN1Bit(name='uplink_tfi_assignment', bit=5),
          CSN1Ref(name='packet_timing_advance', obj=packet_timing_advance_ie),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='packet_extended_timing_advance', bit=2)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='bep_period2', bit=4)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='pfi', bit=7)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='rlc_mode')])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Bit(name='npm_transfer_time', bit=5)])}),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Alt(alt={
              '0': ('', []),
              '1': ('', [
              CSN1Bit(name='reported_timeslots', bit=8),
              CSN1Bit(name='tsh', bit=2)])})])}),
          CSN1Ref(name='uplink_egprs_level', obj=egprs_level_ie),
          CSN1Alt(alt={
            '0': ('', []),
            '1': ('', [
            CSN1Ref(name='pulse_format', obj=pulse_format_ie)])}),
          CSN1Bit(name='indication_of_upper_layer_pdu_start_for_rlc_um'),
          CSN1Bit(name='enhanced_flexible_timeslot_assignment'),
          CSN1Ref(name='egprs_channel_coding_command', obj=egprs_modulation_and_coding_scheme_ie),
          CSN1Ref(obj=padding_bits)])])})])})])])

